//
//  AGThrottlingDispatcher.h
//  AraeliumFoundation
//
//  Created by Seth Willits on 7/12/13.
//  Copyright (c) 2013 Araelium Group. All rights reserved.
//

#import <Foundation/Foundation.h>


// AGThrottlingDispatcher
//
// Dispatches received blocks at most once every 'seconds'.
// 
// If a dispatcher is created with 'seconds' as 1.0, and
// blocks are received at the following times:
//    blockA - 0.0
//    blockB - 0.2
//    blockC - 0.5
//
// blockA will be executed immediately
// blockC will be executed at 1.0 seconds
// 
// Note that blockB is never executed. Its execution is
// first delayed for 0.8 seconds, and by the time it would
// execute, blockC has already been received and thus it
// executes instead.
//

@interface AGThrottlingDispatcher : NSObject
{
    NSTimeInterval _lastTimeFired;
	uint64_t _eventCount;
	dispatch_queue_t _throttlingQueue;
	
	dispatch_queue_t _executionQueue;
	double _throttleSeconds;
}

- (id)initWithQueue:(dispatch_queue_t)queue seconds:(double)seconds;
- (void)dispatch:(dispatch_block_t)block;

@end
